﻿using System;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Api.Controllers
{
    /// <summary>
    /// 医生科研信息
    /// </summary>
    [Route("doctor")]
    [ApiController, Authorize, ApiExplorerSettings(GroupName = "Reviewer")]
    public class ResearchPublicationController : ControllerBase
    {
        private readonly IResearchPublicationService _researchPublicationService;
        public ResearchPublicationController(IResearchPublicationService researchPublicationService)
        {
            _researchPublicationService = researchPublicationService;
        }

        /// <summary>
        /// 查询-医生科学研究信息
        /// </summary>
        /// <param name="doctorId">医生Id</param>
        /// <returns></returns>
        
        [HttpGet, Route("getResearchPublication/{doctorId:guid}")]
        public IResponseOutput<ResearchPublicationDTO> GetDoctorScientificResearch(Guid doctorId)
        {
            return ResponseOutput.Ok(_researchPublicationService.GetResearchPublication(doctorId)) ;
        }

        /// <summary> 添加或跟新医生 科研信息 </summary>
        
        [HttpPost, Route("addResearchPublication")]
        public IResponseOutput AddOrUpdateScientificResearch(
            ResearchPublicationDTO researchPublicationViewModel)
        {
            return _researchPublicationService.AddOrUpdateResearchPublication(researchPublicationViewModel);
        }
    }
}
