﻿using System;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Api.Controllers.Doctor
{
    /// <summary>
    /// 医生休假安排
    /// </summary>
    [Route("holiday")]
    [ApiController, Authorize, ApiExplorerSettings(GroupName = "Reviewer")]
    public class VacationController : ControllerBase
    {
        private readonly IVacationService _vacationService;
        public VacationController(IVacationService vacationService)
        {
            _vacationService = vacationService;
        }
        /// <summary>
        /// 添加休假时间段
        /// </summary>
        /// <param name="vacationViewModel">Status不传</param>
        /// <returns></returns>
        
        [HttpPost, Route("addOrUpdateVacation")]
        public IResponseOutput AddOrUpdateVacation(VacationCommand vacationViewModel)
        {
            return _vacationService.AddOrUpdateVacation(vacationViewModel);
        }
        
        /// <summary>
        /// 删除休假时间段
        /// </summary>
        /// <param name="holidayId">记录Id</param>
        /// <returns></returns>
        
        [HttpDelete, Route("deleteVacation/{holidayId:guid}")]
        public IResponseOutput DeleteVacation(Guid holidayId)
        {
            return _vacationService.DeleteVacation(holidayId);
        }

        /// <summary>
        /// 获取休假时间段列表
        /// </summary>
        /// <returns></returns>
        
        [HttpGet, Route("getVacationList/{doctorId:guid}/{pageIndex:int}/{pageSize:int}")]
        public IResponseOutput<PageOutput<VacationCommand>> GetVacationList(Guid doctorId, int pageIndex, int pageSize)
        {
            return ResponseOutput.Ok(_vacationService.GetVacationList(doctorId, pageIndex, pageSize)) ;
        }
    }
}
