﻿using System;
using System.Collections.Generic;

using IRaCIS.Api.Filter;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels.Pay;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Api.Controllers.Pay
{
    /// <summary>
    /// Financial 支撑信息---医生月度调整
    /// </summary>
    [Route("paymentAdjustment")]
    [ApiController, Authorize, ApiExplorerSettings(GroupName = "Financial")]
    public class PaymentAdjustmentController : ControllerBase
    {
        private readonly IPaymentAdjustmentService _paymentAdjustmentService;

        public PaymentAdjustmentController(IPaymentAdjustmentService paymentAdjustmentService)
        {
            _paymentAdjustmentService = paymentAdjustmentService;
        }

        /// <summary>
        /// 获取费用调整列表
        /// </summary>
        
        [HttpPost, Route("getPaymentAdjustmentList")]
        public IResponseOutput<PageOutput<PaymentAdjustmentDetailDTO>> GetPaymentAdjustmentList(
            PaymentAdjustmentQueryDTO queryParam)
        {
            var result = _paymentAdjustmentService.GetPaymentAdjustmentList(queryParam);
            return ResponseOutput.Ok(result);
        }

        /// <summary>
        /// 添加或更新费用调整[AUTH]
        /// </summary>
        [LogFilter]
        
        [HttpPost, Route("addOrUpdatePaymentAdjustment")]
        public IResponseOutput AddOrUpdatePaymentAdjustment(PaymentAdjustmentCommand addOrUpdateModel)
        {
            return _paymentAdjustmentService.AddOrUpdatePaymentAdjustment(addOrUpdateModel,Guid.Parse(User.FindFirst("id").Value));
        }

        /// <summary>
        /// 删除费用调整记录
        /// </summary>
        
        [LogFilter]
        [HttpDelete, Route("deletePaymentAdjustment/{id:guid}")]
        public IResponseOutput DeleteCostAdjustment(Guid id)
        {
            return _paymentAdjustmentService.DeleteCostAdjustment(id);
        }

        /// <summary>
        /// 获取医生列表
        /// </summary>
        
        [HttpGet, Route("getReviewerSelectList")]
        public IResponseOutput<List<DoctorSelectDTO>> GetReviewerSelectList()
        {
            return ResponseOutput.Ok(_paymentAdjustmentService.GetReviewerSelectList());
        }
    }
}
