﻿using System;
using System.Collections.Generic;

using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;

namespace IRaCIS.Api.Controllers
{
    [Route("sponsor")]
    [ApiController, Authorize, ApiExplorerSettings(GroupName = "Institution")]
    public class SponsorController : ControllerBase
    {
        private readonly ISponsorService _sponsorService;

        public SponsorController(ISponsorService sponsorService)
        {
            _sponsorService = sponsorService;
        }

        /// <summary> 分页获取申办方列表 </summary>
        
        [HttpPost, Route("getSponsorPageList")]
        public IResponseOutput<PageOutput<SponsorDTO>> GetSponsorList(SponsorQueryDTO sponsorSearchModel)
        {
            return ResponseOutput.Ok(_sponsorService.GetSponsorList(sponsorSearchModel));
        }

        /// <summary> 获取所有申办方列表 下拉框 </summary>
        [HttpPost, Route("getAllSponsorList")]
        public IResponseOutput<IEnumerable<SponsorSelectDTO>> GetSponsorSearchList()
        {
            return ResponseOutput.Ok(_sponsorService.GetSponsorSearchList());
        }

        /// <summary> 添加或更新申办方信息 </summary>
        [HttpPost, Route("addOrUpdateSponsor")]
        public IResponseOutput AddSponsor(SponsorCommand addSponsorViewModel)
        {
           return _sponsorService.AddOrUpdateSponsor(addSponsorViewModel);
        }

        /// <summary> 删除申办方信息 </summary>
        [HttpDelete, Route("deleteSponsor/{sponsorId:guid}")]
        public IResponseOutput DeleteSponsor(Guid sponsorId)
        {
            return _sponsorService.DeleteSponsor(sponsorId);
        }
    }
}
