﻿using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using System;
using System.Collections.Generic;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;


namespace IRaCIS.Api.Controllers
{
    /// <summary>
    /// 访视计划
    /// </summary>
    [Route("visitPlan")]
    [ApiController, Authorize, ApiExplorerSettings(GroupName = "Trial")]
    public class VisitPlanController : ControllerBase
    {
        private readonly IVisitPlanService _visitPlanService;
        public VisitPlanController(IVisitPlanService visitPlanService)
        {
            _visitPlanService = visitPlanService;
        }

        /// <summary> 分页获取项目访视计划[New]</summary>
        [HttpPost, Route("getTrialVisitStageList")]
        [ApiExplorerSettings(IgnoreApi = true)]
        public IResponseOutput<PageOutput<VisitStageDTO>> GetTrialVisitStageList(VisitPlanQueryDTO param)
        {
            return ResponseOutput.Ok(_visitPlanService.GetTrialVisitStageList(param));
        }

        /// <summary> 根据项目Id，获取项目访视计划(不分页)[New]</summary>
        [HttpGet, Route("getVisitStageList/{trialId:guid}")]
        public IResponseOutput<IEnumerable<VisitStageDTO>> GetVisitStageList(Guid trialId)
        {
            return ResponseOutput.Ok(_visitPlanService.GetVisitStageList(trialId)) ;
        }
        /// <summary>
        /// 获取访视计划下拉框列表
        /// </summary>
        /// <param name="trialId"></param>
        /// <returns></returns>
        [HttpGet, Route("getTrialVisitStageSelect/{trialId:guid}")]
        public IResponseOutput<IEnumerable<VisitStageSelectDTO>> GetTrialVisitStageSelect(Guid trialId)
        {
            return ResponseOutput.Ok(_visitPlanService.GetTrialVisitStageSelect(trialId));
        }

        /// <summary> 添加或更新访视计划某项[New]</summary>
        [HttpPost, Route("addOrUpdateVisitStage")]
        
        public IResponseOutput AddOrUpdateVisitStage(VisitPlanCommand param)
        {
            return _visitPlanService.AddOrUpdateVisitStage(param);
        }

        /// <summary> 删除项目计划某一项[New]</summary>
        [HttpDelete, Route("deleteVisitStage/{id:guid}")]
        public IResponseOutput DeleteVisitStage(Guid id)
        {
            return _visitPlanService.DeleteVisitStage(id);
        }
    }
}
