﻿using System;
using System.Diagnostics;
using System.Linq;
using System.Threading.Tasks;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using IRaCIS.Core.Domain.Share.AuthUser;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Filters;
using Newtonsoft.Json;

namespace IRaCIS.Api.Filter
{
    public class LogActionFilter : IAsyncActionFilter
    {
        private readonly ILogService _logService;
        private readonly IUserInfo _userInfo;

        public LogActionFilter(ILogService logService, IUserInfo userInfo)
        {
            _logService = logService;
            _userInfo = userInfo;
        }

        public Task OnActionExecutionAsync(ActionExecutingContext context, ActionExecutionDelegate next)
        {

            if (context.ActionDescriptor.EndpointMetadata.Any(m => m.GetType() == typeof(LogFilter)))
            {
                return LogAsync(context, next);
            }
            return next();
        }

        public async Task LogAsync(ActionExecutingContext context, ActionExecutionDelegate next)
        {
            var sw = new Stopwatch();
            sw.Start();

            dynamic actionResult = (await next()).Result;
            sw.Stop();
            var args = JsonConvert.SerializeObject(context.ActionArguments);
            var result = JsonConvert.SerializeObject(actionResult?.Value);

            var attr = (ApiExplorerSettingsAttribute)context.ActionDescriptor.EndpointMetadata.FirstOrDefault(m => m.GetType() == typeof(ApiExplorerSettingsAttribute));
            var groupName = attr?.GroupName;
            var res = actionResult?.Value as IResponseOutput;
            var input = new SystemLogDTO
            {
                ClientIP = string.Empty,
                OptUserId = _userInfo.Id,
                OptUserName = _userInfo.UserName,
                ApiPath = context.ActionDescriptor.AttributeRouteInfo.Template.ToLower(),
                Params = args,
                Result = result,
                RequestTime = DateTime.Now,
                ElapsedMilliseconds = sw.ElapsedMilliseconds,
                Status =res?.IsSuccess?? false,
                Message = res?.ErrorMessage,
                LogCategory = groupName
            };


            _logService.SaveLog2Db(input);
        }
    }
}
