﻿using System;
using System.Net;
using System.Net.Http;
using System.Web.Http.Filters;
using IRaCIS.Common.Logger;
using IRaCIS.Common.Model;

namespace IRaCIS.Api.Filter
{
    public class WebApiExceptionFilterAttribute : ExceptionFilterAttribute
    {
        private ILogger<WebApiExceptionFilterAttribute> _logger;

        public WebApiExceptionFilterAttribute()
        {
            _logger = new NLogLogger<WebApiExceptionFilterAttribute>();
        }



        //public override void OnException(ExceptionContext context)
        public override void OnException(HttpActionExecutedContext context)
        {
            string controllerName = context.ActionContext.ControllerContext.ControllerDescriptor.ControllerName;
            string actionName = context.ActionContext.ActionDescriptor.ActionName;

            _logger.Error("controller=" + controllerName + "  action=" + actionName + "  " + context.Exception.Message);


            if (context.Exception is NotImplementedException)
            {
                context.Response = new HttpResponseMessage(HttpStatusCode.NotImplemented);
            }
            else if (context.Exception is CommonException)
            {
                HttpResponseMessage msg = new HttpResponseMessage(HttpStatusCode.InternalServerError)
                {
                    Content = new StringContent(context.Exception.Message),
                };
                context.Response = msg;

            }
            else if (context.Exception is TimeoutException)
            {
                context.Response = new HttpResponseMessage(HttpStatusCode.RequestTimeout);
            }
            else if (context.Exception is NullReferenceException)
            {
                HttpResponseMessage msg = new HttpResponseMessage(HttpStatusCode.InternalServerError)
                {
                    Content = new StringContent("NullReferenceException"),
                    ReasonPhrase = "An unhandled exception was thrown .controller=" + controllerName + "  action=" + actionName
                };
                context.Response = msg;
            }
            else
            {
                HttpResponseMessage msg = new HttpResponseMessage(HttpStatusCode.InternalServerError)
                {
                    Content = new StringContent(context.Exception.Message),
                    //ReasonPhrase = "An unhandled exception was thrown ." 
                };
                context.Response = msg;
            }

            base.OnException(context);
        }
    }
}