﻿using System;
using System.Collections.Generic;
using EasyCaching.Core.Interceptor;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;

namespace IRaCIS.Application.Interfaces
{
    public interface IDoctorService
    {
        #region  医生 基本信息

        /// <summary>
        /// 基本信息详情展示、编辑使用
        /// </summary>
        /// <param name="doctorId"></param>
        /// <returns></returns>
        DoctorBasicInfoDTO GetDoctorBasicInfo(Guid doctorId);

        /// <summary>
        /// 添加医生基本信息
        /// </summary>
        /// <param name="addBasicInfoParam"></param>
        /// <param name="userId"></param>
        /// <param name="addBasicInfoModel"></param>
        /// <returns></returns>
       IResponseOutput<DoctorBasicInfoCommand>  AddOrUpdateDoctorBasicInfo(DoctorBasicInfoCommand addBasicInfoParam, Guid userId);

        #endregion

        #region 医生 工作信息

        /// <summary>
        /// 获取医生 工作信息
        /// </summary>
        /// <param name="doctorId"></param>
        /// <returns></returns>
        EmploymentDTO GetEmploymentInfo(Guid doctorId);
        /// <summary>
        /// 更新医生 工作信息
        /// </summary>
        /// <param name="updateDoctorWorkInfoViewModel"></param>
        /// <returns></returns>
        IResponseOutput UpdateDoctorEmploymentInfo(EmploymentCommand updateDoctorWorkInfoViewModel);

        #endregion

        /// <summary>
        /// 获取医生技能信息
        /// </summary>
        SpecialtyDTO GetDoctorSpecialty(Guid doctorId);

        /// <summary>
        /// 更新医生技能信息
        /// </summary>
        IResponseOutput UpdateDoctorSpecialty(SpecialtyCommand specialtyUpdateModel);

        /// <summary>
        /// 获取医生 审核状态
        /// </summary>
        ResumeConfirmDTO GetDoctorAuditStatus(Guid doctorId);

        /// <summary>
        /// 审核简历 和合作关系
        /// </summary>
        IResponseOutput AuditResume(ResumeConfirmCommand auditResumeParam,Guid userId);

        /// <summary> 医生详情  入组信息 </summary>
        DoctorEnrollInfoDTO GetDoctorEnrollInfo(Guid doctorId);

        /// <summary>  获取医生参与项目的Sow协议  </summary>
        List<SowDTO> GetDoctorSow(Guid doctorId);

        /// <summary>  获取医生入组的 ack Sow  </summary>
        List<SowDTO> GetDoctorAckSow(Guid doctorId);
    }
}
