﻿using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.Dicom.DTO;
using IRaCIS.Core.Domain.Models;
using System;
using System.Collections.Generic;

namespace IRaCIS.Core.Application.Contracts.DTO
{

    public class QARecordCommand
    {
        public Guid? Id { get; set; }
        public Guid QATemplateId { get; set; }
        public Guid StudyId { get; set; }
        public string Note { get; set; }

        public DateTime? DeadlineTime { get; set; }

    }

    public class QARecordWithUser: QARecordCommand
    {
        public string  UserName { get; set; }
        public DateTime CreateTime { get; set; }

        public int ReplyCount { get; set; }
    }

    public class QADictionaryCommand
    {
     
        public string Note { get; set; }
        public Guid DictionaryId { get; set; }

    }

    public class QADictionaryDTO : QADictionaryCommand
    {
        public string DictionaryValue { get; set; }
    }

    public class QADicView
    {
        public Guid QARecordId { get; set; }
        public string DictionaryValue { get; set; }
        public string Note { get; set; }
        public Guid DictionaryId { get; set; }
    }


    public class QAToalCommand
    {
        public Guid StudyId { get; set; }
        public Guid TrialId { get; set; }
        public Guid SiteId { get; set; }
        public QARecordCommand QARecord { get; set; }

        public List<QADictionaryCommand> QADictionaryList { get; set; } = new List<QADictionaryCommand>();

    }

    public class QADTO
    {
        public List<QARecordDTO> QARecordList { get; set; } = new List<QARecordDTO>();
        public StudySubjectInfoDTO StudySubjectInfo { get; set; }
        public AcquisitionSpecificationDTO Specification { get; set; }
    }

    public class QARecordDTO
    {
        public QARecordWithUser QARecord { get; set; }

        public List<QADicView> QADictionaryList { get; set; } = new List<QADicView>();

    }

    public class StudySubjectInfoDTO
    {
        public int StudyStatus { get; set; }
        public string SubjectCode { get; set; }
        public string SubjectName { get; set; }
        public int SubjectAge { get; set; }
        public string SubjectSex { get; set; }

        public string PatientName { get; set; } = string.Empty;
        public string PatientAge { get; set; } = string.Empty;
        public string PatientSex { get; set; } = string.Empty;

        public string Modalities { get; set; }

        public int SeriesCount { get; set; }

        public int InstanceCount { get; set; }

        public string Comment { get; set; }
        public string QAComment { get; set; }
        public bool Anonymize { get; set; }


        public decimal VisitNum { get; set; }
        public string VisitName { get; set; } = string.Empty;
        public DateTime? SVSTDTC { get; set; }
        public DateTime? SVENDTC { get; set; }
        public bool InPlan { get; set; }


    }
    public class QADialogCommand
    {
        public string TalkContent { get; set; }
        public Guid QARecordId { get; set; }
        public Guid? ParentId { get; set; }
    }


    public class DialogDTO
    {
        public Guid Id { get; set; }
        public string TalkContent { get; set; }
        //public Guid QARecordId { get; set; }
        public string From { get; set; }
        public string To { get; set; }
        public Guid UserId { get; set; }

        public string PreviousContent { get; set; }

        public DateTime CreateTime { get; set; }
    }

    public class DialogNode
    {
        public Guid Id { get; set; }
        public Guid ParentId { get; set; }
        public Guid UserId { get; set; }
        public string UserName { get; set; }

        public string TalkContent { get; set; }
        public Guid QARecordId { get; set; }

        public DateTime CreateTime { get; set; }
    }

}