﻿using AutoMapper;
using AutoMapper.QueryableExtensions;
using IRaCIS.Application.ExpressionExtend;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using IRaCIS.Core.Domain.Interfaces;
using IRaCIS.Core.Domain.Models;
using IRaCIS.Core.Domain.Share;
using IRaCIS.Infra.Data.ExpressionExtend;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Dynamic.Core;
using System.Linq.Expressions;


namespace IRaCIS.Application.Services
{
    public class DoctorListQueryService : IDoctorListQueryService
    {
        private readonly IDoctorRepository _doctorRepository;
        private readonly IDictionaryRepository _dictionaryRepository;
        private readonly IHospitalRepository _hospitalRepository;

        private readonly IEnrollRepository _enrollRepository;
        private readonly IEnrollDetailRepository _enrollDetailRepository;
        private readonly IDoctorDictionaryRepository _doctorDictionaryRepository;
        private readonly IUserRepository _userRepository;
        private readonly IAttachmentRepository _attachmentRepository;
        private readonly IUserDoctorRepository _userDoctorRepository;
        private readonly ITrialRepository _trialRepository;
        private readonly ITrialPaymentPriceRepository _TrialPaymentPrice;
        private readonly IMapper _mapper;

        public DoctorListQueryService(IDoctorRepository doctorInfoRepository,
            IDictionaryRepository dictionaryRepository, IHospitalRepository hospitalRepository,
            IEnrollRepository intoGroupRepository, IEnrollDetailRepository intoGroupDetailRepository,
            IDoctorDictionaryRepository doctorDictionaryRepository, IUserRepository userRepository,
            IAttachmentRepository attachmentRepository,
            IUserDoctorRepository userDoctorRepository,
            ITrialRepository trialRepository,
            ITrialPaymentPriceRepository trialPaymentPrice,
            IMapper mapper
          )
        {
            _doctorRepository = doctorInfoRepository;
            _dictionaryRepository = dictionaryRepository;
            _hospitalRepository = hospitalRepository;
            _TrialPaymentPrice = trialPaymentPrice;

            _enrollRepository = intoGroupRepository;
            _enrollDetailRepository = intoGroupDetailRepository;
            _doctorDictionaryRepository = doctorDictionaryRepository;
            _userRepository = userRepository;
            _attachmentRepository = attachmentRepository;
            _userDoctorRepository = userDoctorRepository;
            _trialRepository = trialRepository;
            _mapper = mapper;
        }

        /// <summary>
        /// Reviewer列表分页查询
        /// </summary>
        public PageOutput<DoctorDTO> GetReviewerList(DoctorSearchDTO param, Guid reviewerId)
        {
            // 医生Lambda表达式树 条件拼接
            Expression<Func<Doctor, bool>> doctorLambda = default;

            //医生信息 + 医院信息  + 三个统计信息 + 医生表搜索条件过滤 + 中间表条件过滤
            IQueryable<DoctorDTO> doctorQueryable = default;

            //多选记录条数--  用于having过滤
            var realCount = param.ReadingTypeIdList.Count + param.TitleIdList.Count +
                            param.SubspecialityIdList.Count;

            var user = _userRepository.GetAll().First(t => t.Id == reviewerId);

            if (user.SuperAdmin || user.UserType.Contains(StaticData.DataAdmin) || user.UserType.Contains(StaticData.CROCoordinator))
            {
                if (user.UserType.Contains(StaticData.CROCoordinator))//CRO协调者
                {
                    if (user.OrganizationName== "Parexel")
                    {
                        param.Nation = 0;//中国医生
                    }
                    //else if（）太美只能查看国外医生
                    param.ContractorStatus = (int)ContractorStatus.Cooperation;
                    param.InformationConfirmed = (int)ReviewerInformationConfirmStatus.ConfirmPass;
                }
                doctorLambda = GetDoctorSearchLambda(param);
            }
            else
            {
                doctorLambda = GetDoctorSearchLambda(param);

                IQueryable<Guid> userFilter = _userDoctorRepository.GetAll()
                    .Where(f => f.UserId == reviewerId).Select(k => k.DoctorId);

                doctorLambda = doctorLambda.And(t =>
                    userFilter.Contains(t.Id));
            }

            //有多选信息过滤
            if (realCount > 0)
            {
                var doctorDictionaryFilterQueryable = GetDoctorOneToManyDictionaryFilterQueryable(param);

                doctorLambda = doctorLambda.And(t => doctorDictionaryFilterQueryable.Contains(t.Id));
            }
            doctorQueryable = GetDoctorQueryable(doctorLambda);

            ////是否入组 过滤（统计字段）
            if (param.EnrollStatus != null)
            {
                if (param.EnrollStatus == (int)ReviewerEnrollStatus.Yes)
                {
                    doctorQueryable = doctorQueryable.Where(o => o.Reading == 0);
                }
                else
                {
                    doctorQueryable = doctorQueryable.Where(o => o.Reading > 0);
                }
            }

            var propName = param.SortField == string.Empty ? "CreateTime" : param.SortField;
            
            doctorQueryable = param.Asc
                    ? doctorQueryable.OrderBy(propName).ThenBy(t=>t.Code)
                    : doctorQueryable.OrderByDescending(propName).ThenBy(t => t.Code);

            if (propName == "FirstName" || propName == "LastName")
            {
                doctorQueryable = param.Asc
                    ? doctorQueryable.OrderBy(t => t.LastName).ThenBy(t => t.FirstName)
                    : doctorQueryable.OrderByDescending(t => t.LastName).ThenBy(t => t.FirstName);
            }
            var count = doctorQueryable.Count();
            doctorQueryable = doctorQueryable
                .Skip((param.PageIndex - 1) * param.PageSize)
                .Take(param.PageSize);
            var doctorList = doctorQueryable.ToList();

            var doctorResumeIdList = _attachmentRepository.Find(u => u.Type == "Resume" && u.IsOfficial).Select(u => u.DoctorId).ToList();
            foreach (var item in doctorList)
            {
                if (doctorResumeIdList.Contains(item.Id))
                {
                    item.HasResume = true;
                }
            }

            DealDoctorDictionaryInfo(doctorList);

            return new PageOutput<DoctorDTO>(param.PageIndex,
                param.PageSize, count, doctorList);
        }





        public  PageOutput<EnrollViewModel> GetTrialDoctorList(EnrollGetQuery challengeQuery)
        {
            var costStatisticsQueryable = from enroll in _enrollRepository.GetAll().Where(t => t.TrialId == challengeQuery.TrialId)
                                          join dociorc in _doctorRepository.GetAll() on enroll.DoctorId equals dociorc.Id
                                          join price in _TrialPaymentPrice.GetAll() on enroll.TrialId equals price.TrialId
                                          select new EnrollViewModel()
                                          {
                                              ChineseName = dociorc.ChineseName,
                                              AdjustmentMultiple = enroll.AdjustmentMultiple,
                                              FirstName = dociorc.FirstName,
                                              LastName = dociorc.LastName,
                                              DoctorId = dociorc.Id,
                                              TrialId = challengeQuery.TrialId,
                                              Id = enroll.Id,
                                              Training = enroll.Training,
                                              RefresherTraining = enroll.RefresherTraining,
                                              Timepoint = enroll.Timepoint,
                                              Timepoint48H = enroll.Timepoint48H,
                                              Timepoint24H = enroll.Timepoint24H,
                                              Adjudication = enroll.Adjudication,
                                              Adjudication48H = enroll.Adjudication48H,
                                              Adjudication24H = enroll.Adjudication24H,
                                              Global = enroll.Global,
                                              Code = dociorc.Code,
                                              ReviewerCode = dociorc.Code,
                                              Downtime = enroll.Downtime,
                                          };


            return new PageOutput<EnrollViewModel>(challengeQuery.PageIndex,
               challengeQuery.PageSize, costStatisticsQueryable.Count(),
                costStatisticsQueryable.Skip((challengeQuery.PageIndex - 1) * challengeQuery.PageSize).Take(challengeQuery.PageSize).ToList()
               );
           




        }



        public bool  AddOrUpdateEnroll(EnrollCommand addOrUpdateModel)
        {

            var trialDoctoritem = _enrollRepository.GetAll().FirstOrDefault(u => u.Id == addOrUpdateModel.Id);
            if (trialDoctoritem == null)//insert
            {
                var trialExt = _mapper.Map<Enroll>(addOrUpdateModel);
                _enrollRepository.Add(trialExt);
            }
            else//update
            {
                _enrollRepository.Update(_mapper.Map(addOrUpdateModel, trialDoctoritem));
            }
            return  _enrollRepository.SaveChanges();
        }


        #region 入组查询相关
        /// <summary>
        /// 获取可筛选筛选及已经筛选的医生列表
        /// </summary>
        public PageOutput<SelectionReviewerDTO> GetSelectionReviewerList(
            ReviewerSelectionQueryDTO selectionQueryParam)
        {
            var trial = _trialRepository.FindSingleOrDefault(s => s.Id == selectionQueryParam.TrialId);

            // 如果是CN&Abroad 为2   就不筛选

            if (trial.AttendedReviewerType != 2)
            {
                selectionQueryParam.Nation = trial.AttendedReviewerType;
            }
          
            var dic = GetDictionary();
            //医生条件搜索Lambda拼接
            Expression<Func<Doctor, bool>> doctorLambda = GetDoctorSearchLambda(selectionQueryParam); ;

            //记录条数--having过滤
            var realCount = selectionQueryParam.ReadingTypeIdList.Count + selectionQueryParam.TitleIdList.Count +
                            selectionQueryParam.SubspecialityIdList.Count;

            IQueryable<DoctorDTO> doctorQueryable = default;

            if (realCount != 0)//多选查询的
            {
                var doctorDictionaryFilterQueryable = GetDoctorOneToManyDictionaryFilterQueryable(selectionQueryParam);

                doctorLambda = doctorLambda.And(t => doctorDictionaryFilterQueryable.Contains(t.Id));
                doctorQueryable = GetDoctorQueryable(doctorLambda);
            }
            else
            {
                doctorQueryable = GetDoctorQueryable(doctorLambda);
            }
            var propName = selectionQueryParam.SortField == "" ? "Code" : selectionQueryParam.SortField;
            doctorQueryable = selectionQueryParam.Asc
                ? DynamicQueryableExtensions.OrderBy(doctorQueryable, propName)
                : DynamicQueryableExtensions.OrderBy(doctorQueryable, propName + " desc");
            if (propName == "FirstName" || propName == "LastName")
            {
                doctorQueryable = selectionQueryParam.Asc
              ? doctorQueryable.OrderBy(t => t.LastName).ThenBy(t => t.FirstName)
              : doctorQueryable.OrderByDescending(t => t.LastName).ThenBy(t => t.FirstName);
            }

            if (selectionQueryParam.EnrollStatus != null)
            {
                if (selectionQueryParam.EnrollStatus == (int)ReviewerEnrollStatus.Yes)
                {
                    doctorQueryable = doctorQueryable.Where(o => o.Reading == 0);
                }
                else
                {
                    doctorQueryable = doctorQueryable.Where(o => o.Reading > 0);
                }
            }
            var count = doctorQueryable.Count();
            doctorQueryable = doctorQueryable
                .Skip((selectionQueryParam.PageIndex - 1) * selectionQueryParam.PageSize)
                .Take(selectionQueryParam.PageSize);
            var doctorViewList = doctorQueryable.ToList();

            DealDoctorDictionaryInfo(doctorViewList);
            var doctorList = _mapper.Map<List<SelectionReviewerDTO>>(doctorViewList);

            //处理字典表信息连接、判断是否已申请时间 申请人
            doctorList = DealScreeningDoctorsConnectInfo(doctorList, dic, selectionQueryParam);

            return new PageOutput<SelectionReviewerDTO>(selectionQueryParam.PageIndex,
                selectionQueryParam.PageSize, count, doctorList);

        }

        /// <summary>
        /// 获取提交CRO或者CRO审核的Reviewer列表
        /// </summary>
        public PageOutput<ConfirmationReviewerDTO> GetSubmissionOrApprovalReviewerList(
            ReviewerSubmissionQueryDTO param)
        {
            //入组状态筛选条件
            var intoGroupSearchState = param.IntoGroupSearchState;
            var trailId = param.TrialId;

            Expression<Func<EnrollDetail, bool>> intoGroupDetailLambda = x => x.TrialId == trailId;

            Expression<Func<Enroll, bool>> intoGroupLambda = x => x.TrialId == trailId;
            //提交CRO  以及下载简历列表  
            if (intoGroupSearchState == 1)
            {

                intoGroupLambda = intoGroupLambda.And(t => t.EnrollStatus >=
                                                           (int)EnrollStatus.HasApplyDownloadResume);

                intoGroupDetailLambda = intoGroupDetailLambda.And(t => t.EnrollStatus ==
                                                                       (int)EnrollStatus.HasCommittedToCRO);
            }

            //CRO确认列表  状态为 已提交CRO
            else if (intoGroupSearchState == 4)
            {
                intoGroupLambda = intoGroupLambda.And(t => t.EnrollStatus >=
                                                           (int)EnrollStatus.HasCommittedToCRO);
                intoGroupDetailLambda = intoGroupDetailLambda.And(t => t.EnrollStatus ==
                                                                       (int)EnrollStatus.InviteIntoGroup);
            }

            //医生表 表达式树条件 过滤
            var doctorQueryable = from doctorItem in _doctorRepository.Find()
                                      //连接医院信息 左连接
                                  join hospitalItem in _hospitalRepository.GetAll() on doctorItem.HospitalId
                                      equals hospitalItem.Id
                                      into gt
                                  from hospital in gt.DefaultIfEmpty()
                                  join intoGroup in _enrollRepository.GetAll().Where(intoGroupLambda)
                                      on doctorItem.Id equals intoGroup.DoctorId
                                  select new ConfirmationReviewerDTO()
                                  {
                                      //医院信息
                                      City =  hospital.City ,
                                      HospitalName = hospital.HospitalName ,
                                      Country = hospital.Country ,

                                      //医生各种基本信息+单选外键
                                      Id = doctorItem.Id,
                                      FirstName = doctorItem.FirstName,
                                      LastName = doctorItem.LastName,
                                      ChineseName = doctorItem.ChineseName,
                                      Code = doctorItem.Code
                                  };

            var propName = param.SortField == "" ? "Code" : param.SortField;
            doctorQueryable = param.Asc
                ? DynamicQueryableExtensions.OrderBy(doctorQueryable, propName)
                : DynamicQueryableExtensions.OrderBy(doctorQueryable, propName + " desc");
            if (propName == "FirstName" || propName == "LastName")
            {
                doctorQueryable = param.Asc
              ? doctorQueryable.OrderBy(t => t.LastName).ThenBy(t => t.FirstName)
              : doctorQueryable.OrderByDescending(t => t.LastName).ThenBy(t => t.FirstName);
            }

            var count = doctorQueryable.Count();

            doctorQueryable = doctorQueryable
                .Skip((param.PageIndex - 1) * param.PageSize)
                .Take(param.PageSize);

            var doctorViewList = doctorQueryable.ToList();

            var doctorStateList = (from intoGroupDetail in _enrollDetailRepository.GetAll().Where(intoGroupDetailLambda)
                                   join userItem in _userRepository.GetAll() on intoGroupDetail.CreateUserId equals userItem.Id into tt
                                   from user in tt.DefaultIfEmpty()
                                   select new DoctorStateModelDTO()
                                   {
                                       DoctorId = intoGroupDetail.DoctorId,
                                       IntoGroupState = intoGroupDetail.EnrollStatus,
                                       OptTime = intoGroupDetail.CreateTime,
                                       OptUserName = user == null ? "" : user.UserName
                                   }).ToList();

            if (intoGroupSearchState == 1)
            {

                foreach (var doctorIntoGroupOptViewModel in doctorViewList)
                {
                    var opt = doctorStateList.FirstOrDefault(t => t.DoctorId == doctorIntoGroupOptViewModel.Id);
                    if (opt != null)
                    {
                        doctorIntoGroupOptViewModel.DoctorTrialState = (int)EnrollStatus.HasCommittedToCRO;
                        doctorIntoGroupOptViewModel.OptTime = opt.OptTime;
                        doctorIntoGroupOptViewModel.OptUserName = opt.OptUserName;
                    }
                }
            }

            //CRO确认列表  状态为 已提交CRO
            else if (intoGroupSearchState == 4)
            {
                foreach (var doctorIntoGroupOptViewModel in doctorViewList)
                {
                    var opt = doctorStateList.FirstOrDefault(t => t.DoctorId == doctorIntoGroupOptViewModel.Id);
                    if (opt != null)
                    {
                        doctorIntoGroupOptViewModel.DoctorTrialState = (int)EnrollStatus.InviteIntoGroup;
                        doctorIntoGroupOptViewModel.OptTime = opt.OptTime;
                        doctorIntoGroupOptViewModel.OptUserName = opt.OptUserName;
                    }

                }
            }

            return new PageOutput<ConfirmationReviewerDTO>(param.PageIndex,
                param.PageSize, count, doctorViewList);
        }

        /// <summary>
        /// 获取项目入组待确认及已经确认或拒绝的Reviewer 列表
        /// </summary>
        public PageOutput<ConfirmationReviewerDTO> GetConfirmationReviewerList(
            ReviewerConfirmationQueryDTO trialIdPageModel)
        {

            Expression<Func<Enroll, bool>> intoGroupLambda = x => x.TrialId == trialIdPageModel.TrialId && x.EnrollStatus >= (int)EnrollStatus.InviteIntoGroup;

            var doctorQueryable = from doctor in _doctorRepository.Find()
                                      //连接医院信息 左连接
                                  join hospitalItem in _hospitalRepository.GetAll() on doctor.HospitalId
                                      equals hospitalItem.Id
                                      into gt
                                  from hospital in gt.DefaultIfEmpty()
                                  join stateItem in _enrollRepository.Find(intoGroupLambda) on
                                      doctor.Id equals stateItem.DoctorId
                                  select new ConfirmationReviewerDTO()
                                  {
                                      //医院信息
                                      City = hospital.City,
                                      HospitalName = hospital.HospitalName,
                                      Country = hospital.Country,

                                      //医生各种基本信息+单选外键
                                      Id = doctor.Id,
                                      FirstName = doctor.FirstName,
                                      LastName = doctor.LastName,
                                      ChineseName = doctor.ChineseName,
                                      Code = doctor.Code,

                                      DoctorTrialState = (int)EnrollStatus.InviteIntoGroup
                                  };
            var propName = trialIdPageModel.SortField == "" ? "Code" : trialIdPageModel.SortField;
            doctorQueryable = trialIdPageModel.Asc
                ? DynamicQueryableExtensions.OrderBy(doctorQueryable, propName)
                : DynamicQueryableExtensions.OrderBy(doctorQueryable, propName + " desc");
            if (propName == "FirstName" || propName == "LastName")
            {
                doctorQueryable = trialIdPageModel.Asc
                    ? doctorQueryable.OrderBy(t => t.LastName).ThenBy(t => t.FirstName)
                    : doctorQueryable.OrderByDescending(t => t.LastName).ThenBy(t => t.FirstName);
            }
            var count = doctorQueryable.Count();
            doctorQueryable = doctorQueryable
                .Skip((int)((trialIdPageModel.PageIndex - 1) * trialIdPageModel.PageSize))
                .Take((int)trialIdPageModel.PageSize);
            var doctorList = doctorQueryable.ToList();

            Expression<Func<EnrollDetail, bool>> intoGroupDetailLambda = x => x.TrialId == trialIdPageModel.TrialId && x.EnrollStatus > (int)EnrollStatus.InviteIntoGroup;

            var doctorStateList = (from intoGroupDetail in _enrollDetailRepository.GetAll().Where(intoGroupDetailLambda)
                                   join userItem in _userRepository.GetAll() on intoGroupDetail.CreateUserId equals userItem.Id into tt
                                   from user in tt.DefaultIfEmpty()
                                   select new DoctorStateModelDTO()
                                   {
                                       DoctorId = intoGroupDetail.DoctorId,
                                       IntoGroupState = intoGroupDetail.EnrollStatus,
                                       OptTime = intoGroupDetail.CreateTime,
                                       OptUserName = user == null ? "" : user.UserName
                                   }).ToList();

            doctorList.ForEach(doctor =>
            {
                var doctorState2 = doctorStateList.FirstOrDefault(t => t.DoctorId == doctor.Id &&
                                                                       t.IntoGroupState ==
                                                                       (int)EnrollStatus
                                                                           .RefuseIntoGroup);
                if (doctorState2 != null)
                {
                    doctor.DoctorTrialState = (int)EnrollStatus.RefuseIntoGroup;
                    doctor.OptTime = doctorState2.OptTime;
                    doctor.OptUserName = doctorState2.OptUserName;
                }

                var doctorState3 = doctorStateList.FirstOrDefault(t => t.DoctorId == doctor.Id &&
                                                                      t.IntoGroupState ==
                                                                      (int)EnrollStatus
                                                                          .ConfirmIntoGroup);
                if (doctorState3 != null)
                {
                    doctor.DoctorTrialState = (int)EnrollStatus.ConfirmIntoGroup;
                    doctor.OptTime = doctorState3.OptTime;
                    doctor.OptUserName = doctorState3.OptUserName;
                }

            });

            return new PageOutput<ConfirmationReviewerDTO>(trialIdPageModel.PageIndex, trialIdPageModel.PageSize, count, doctorList);
        }

        /// <summary>
        /// 处理是否已经筛选过 以及筛选时间  筛选人
        /// </summary>
        private List<SelectionReviewerDTO> DealScreeningDoctorsConnectInfo(List<SelectionReviewerDTO> doctorList, List<DicViewModelDTO> dic, ReviewerSelectionQueryDTO searchModel)
        {
            Expression<Func<EnrollDetail, bool>> intoGroupDetailLambda = x => x.TrialId == searchModel.TrialId && x.EnrollStatus ==
                                                                                 (int)EnrollStatus.HasApplyDownloadResume;
            var doctorStateList = (from intoGroupDetail in _enrollDetailRepository.GetAll().Where(intoGroupDetailLambda)
                                   join userItem in _userRepository.GetAll() on intoGroupDetail.CreateUserId equals userItem.Id into tt
                                   from user in tt.DefaultIfEmpty()
                                   select new DoctorStateModelDTO()
                                   {
                                       DoctorId = intoGroupDetail.DoctorId,
                                       IntoGroupState = intoGroupDetail.EnrollStatus,
                                       OptTime = intoGroupDetail.CreateTime,
                                       OptUserName = user.UserName
                                   }).ToList();
            // 处理关联字典表的逻辑
            doctorList.ForEach(
                doctor =>
                {
                    //简历申请列表  --处理已经申请的
                    var doctorState = doctorStateList.FirstOrDefault(t => t.DoctorId == doctor.Id &&
                                                                          t.IntoGroupState ==
                                                                          (int)EnrollStatus.HasApplyDownloadResume);
                    if (doctorState != null)
                    {
                        doctor.DoctorTrialState = (int)EnrollStatus.HasApplyDownloadResume;
                        doctor.OptTime = doctorState.OptTime;
                        doctor.OptUserName = doctorState.OptUserName;
                    }
                });
            return doctorList;

        }

        #endregion

        /// <summary>
        /// 连接医生关联字典表信息
        /// </summary>
        private void DealDoctorDictionaryInfo(List<DoctorDTO> doctorList)
        {
            var doctorHasSearchedIdList = doctorList.Select(t => t.Id).ToList();

            var query = from doctorId in doctorHasSearchedIdList
                        join doctorDic in _doctorDictionaryRepository.GetAll() on doctorId equals doctorDic.DoctorId
                        join dic in _dictionaryRepository.GetAll() on doctorDic.DictionaryId equals dic.Id
                        select new
                        {
                            DoctorId = doctorId,
                            Id = dic.Id,
                            keyName = dic.KeyName,
                            Value = dic.Value,
                            ShowOrder = dic.ShowOrder
                        };

            //获取多选信息列表
            var doctorMultipleInfoList = query.ToList();
            //字典表 后面做缓存
            var dictionaryList = GetDictionary();

            doctorList.ForEach(doctor =>
            {
                doctor.SubspecialityIds = doctorMultipleInfoList.Where(t => t.keyName == StaticData.Subspeciality && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Id).ToList();
                doctor.SubspecialityList = doctorMultipleInfoList.Where(t => t.keyName == StaticData.Subspeciality && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Value).ToList();

                doctor.TitleIdList = doctorMultipleInfoList.Where(t => t.keyName == StaticData.Title && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Id).ToList();
                doctor.TitleList = doctorMultipleInfoList.Where(t => t.keyName == StaticData.Title && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Value).ToList();

                doctor.ReadingTypeIds = doctorMultipleInfoList.Where(t => t.keyName == StaticData.ReadingType && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Id).ToList();
                doctor.ReadingTypeList = doctorMultipleInfoList.Where(t => t.keyName == StaticData.ReadingType && t.DoctorId == doctor.Id)
                    .OrderBy(t => t.ShowOrder).Select(t => t.Value).ToList();
                doctor.Department = doctor.DepartmentId == Guid.Empty
                    ? doctor.DepartmentOther ?? string.Empty
                    : dictionaryList.FirstOrDefault(o => o.Id == doctor.DepartmentId)?.Value ?? string.Empty;

                doctor.Speciality = doctor.SpecialityId == Guid.Empty
                    ? doctor.SpecialityOther ?? string.Empty
                    : dictionaryList.FirstOrDefault(o => o.Id == doctor.SpecialityId)?.Value ?? string.Empty;

                doctor.Rank = doctor.RankId == Guid.Empty
                    ? doctor.RankOther ?? string.Empty
                    : dictionaryList.FirstOrDefault(o => o.Id == doctor.RankId)?.Value ?? string.Empty;

                doctor.Position = doctor.PositionId == Guid.Empty
                    ? doctor.PositionOther ?? string.Empty
                    : dictionaryList.FirstOrDefault(o => o.Id == doctor.PositionId)?.Value ?? string.Empty;
            });
        }
        #region 获取医生列表Linq 语句
        private IQueryable<Guid> GetDoctorOneToManyDictionaryFilterQueryable(DoctorSearchDTO doctorSearchModel)
        {
            //多选项 多选  组合查询  中间表过滤
            Expression<Func<DoctorDictionary, bool>> doctorDictionaryLambda = x => true;
            var dicGuidList = new List<Guid>();
            dicGuidList.AddRange(doctorSearchModel.ReadingTypeIdList);
            dicGuidList.AddRange(doctorSearchModel.SubspecialityIdList);
            dicGuidList.AddRange(doctorSearchModel.TitleIdList);

            if (dicGuidList.Count > 0)
            {
                doctorDictionaryLambda = doctorDictionaryLambda.And(u => dicGuidList.Contains(u.DictionaryId));
            }
            var realCount = doctorSearchModel.ReadingTypeIdList.Count + doctorSearchModel.TitleIdList.Count +
                            doctorSearchModel.SubspecialityIdList.Count;

            var doctorDictionaryQueryable = from doctorDicItem in _doctorDictionaryRepository.GetAll().Where(doctorDictionaryLambda)
                                            group doctorDicItem by doctorDicItem.DoctorId
                into g
                                            select new
                                            {
                                                DoctorId = g.Key,
                                                Count = g.Count()
                                            }
                into zz
                                            where zz.Count == realCount
                                            select zz.DoctorId;

            return doctorDictionaryQueryable;
        }

        private IQueryable<DoctorDTO> GetDoctorQueryable(Expression<Func<Doctor, bool>> doctorLambda)
        {
            //医生表 表达式树条件 过滤
            var doctorQueryable = from doctorItem in _doctorRepository.Find(doctorLambda)
                                      //连接医院信息 左连接
                                  join hospitalItem in _hospitalRepository.GetAll() on doctorItem.HospitalId
                                         equals hospitalItem.Id
                                         into gt
                                  from hospital in gt.DefaultIfEmpty()
                                      //获取统计信息--分组聚合
                                  join statisticsItem in (
                                          from intoGroupItem in _enrollRepository.GetAll()
                                          group intoGroupItem by intoGroupItem.DoctorId
                                          into g
                                          select new
                                          {
                                              DoctorId = g.Key,
                                              Submitted = g.Sum(t =>
                                                  t.EnrollStatus == (int)EnrollStatus.HasCommittedToCRO ? 1 : 0),
                                              Approved = g.Sum(t =>
                                                  t.EnrollStatus == (int)EnrollStatus.InviteIntoGroup ? 1 : 0),
                                              Reading = g.Sum(t =>
                                                  t.EnrollStatus == (int)EnrollStatus.DoctorReading ? 1 : 0)
                                          }) on doctorItem.Id equals statisticsItem.DoctorId
                                      into gj
                                  from sub in gj.DefaultIfEmpty()
                                  select new DoctorDTO()
                                  {
                                      //医院信息
                                      City =  hospital.City,
                                      HospitalName = hospital.HospitalName,
                                      Country = hospital.Country,
                                      //医生各种基本信息+单选外键
                                      Id = doctorItem.Id,
                                      FirstName = doctorItem.FirstName,
                                      LastName = doctorItem.LastName,
                                      ChineseName = doctorItem.ChineseName,
                                      Code = doctorItem.Code,
                                      Phone = doctorItem.Phone,
                                      Introduction = doctorItem.Introduction,
                                      EMail = doctorItem.EMail,
                                      WeChat = doctorItem.WeChat,
                                      DepartmentId = doctorItem.DepartmentId,
                                      DepartmentOther = doctorItem.DepartmentOther,
                                      SpecialityId = doctorItem.SpecialityId,
                                      SpecialityOther = doctorItem.SpecialityOther,
                                      RankId = doctorItem.RankId,
                                      RankOther = doctorItem.RankOther,
                                      PositionId = doctorItem.PositionId,
                                      PositionOther = doctorItem.PositionOther,
                                      HospitalId = doctorItem.HospitalId,
                                      HospitalOther = doctorItem.HospitalOther,
                                      ReadingTypeOther = doctorItem.ReadingTypeOther,
                                      SubspecialityOther = doctorItem.SubspecialityOther,

                                      GCP = doctorItem.GCP,
                                      GCPId = doctorItem.GCPId,
                                      ResumePath = doctorItem.ResumePath,
                                      ResumeStatus = doctorItem.ResumeStatus,
                                      CooperateStatus = doctorItem.CooperateStatus,
                                      CreateTime = doctorItem.CreateTime,
                                      Reconfirmed = doctorItem.ReviewStatus == 1,
                                      //统计信息
                                      Approved = sub.Approved,
                                      Reading = sub.Reading,
                                      Submitted = sub.Submitted
                                  };
            return doctorQueryable;
        }


        #endregion

        #region 获取字典表、连接字典表 、表达式条件拼接

        //获取字典表
        private List<DicViewModelDTO> GetDictionary()
        {
            return //字典表 后面做缓存
                _dictionaryRepository.GetAll().ProjectTo<DicViewModelDTO>(_mapper.ConfigurationProvider).OrderBy(t => t.KeyName)
                    .ThenBy(t => t.ShowOrder).ToList();
        }

        //医生条件查询拼接
        private Expression<Func<Doctor, bool>> GetDoctorSearchLambda(DoctorSearchDTO doctorSearchModel)
        {
            Expression<Func<Doctor, bool>> doctorLambda = x => true;
            if (doctorSearchModel.DepartmentId != Guid.Empty && doctorSearchModel.DepartmentId != null)
            {
                doctorLambda = doctorLambda.And(o => o.DepartmentId == doctorSearchModel.DepartmentId);
            }

            if (doctorSearchModel.SpecialityId != Guid.Empty && doctorSearchModel.SpecialityId != null)
            {
                doctorLambda = doctorLambda.And(o => o.SpecialityId == doctorSearchModel.SpecialityId);
            }

            if (doctorSearchModel.HospitalId != Guid.Empty && doctorSearchModel.HospitalId != null)
            {
                doctorLambda = doctorLambda.And(o => o.HospitalId == doctorSearchModel.HospitalId);
            }

            if (doctorSearchModel.PositionId != Guid.Empty && doctorSearchModel.PositionId != null)
            {
                doctorLambda = doctorLambda.And(o => o.PositionId == doctorSearchModel.PositionId);
            }

            if (doctorSearchModel.RankId != Guid.Empty && doctorSearchModel.RankId != null)
            {
                doctorLambda = doctorLambda.And(o => o.RankId == doctorSearchModel.RankId);
            }

            if (doctorSearchModel.ContractorStatus != null)
            {
                doctorLambda = doctorLambda.And(o => o.CooperateStatus == doctorSearchModel.ContractorStatus);
            }

            if (doctorSearchModel.InformationConfirmed != null)
            {
                doctorLambda = doctorLambda.And(o => o.ResumeStatus == doctorSearchModel.InformationConfirmed);
            }
            if (doctorSearchModel.AcceptingNewTrial != null)
            {
                doctorLambda = doctorLambda.And(o => o.AcceptingNewTrial == doctorSearchModel.AcceptingNewTrial);
            }
            if (doctorSearchModel.ActivelyReading != null)
            {
                doctorLambda = doctorLambda.And(o => o.ActivelyReading == doctorSearchModel.ActivelyReading);
            }

            if (!string.IsNullOrWhiteSpace(doctorSearchModel.Name))
            {
                var doctorName = doctorSearchModel.Name.Trim();
                doctorLambda = doctorLambda.And(o => o.LastName.Contains(doctorName)
                    || o.FirstName.Contains(doctorName)
                    || o.ChineseName.Contains(doctorName));
            }
            if (doctorSearchModel.Nation != null) //2全部
            {
                doctorLambda = doctorLambda.And(o => o.Nation == doctorSearchModel.Nation);
            }

            return doctorLambda;
        }
        #endregion
    }
}

