﻿using AutoMapper;
using AutoMapper.QueryableExtensions;
using IRaCIS.Application.ExpressionExtend;
using IRaCIS.Application.Interfaces;
using IRaCIS.Application.ViewModels;
using IRaCIS.Core.Application.Contracts.RequestAndResponse;
using IRaCIS.Core.Domain.Interfaces;
using IRaCIS.Core.Domain.Models;
using IRaCIS.Infra.Data.ExpressionExtend;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Linq.Expressions;


namespace IRaCIS.Application.Services
{
    public class VisitStageService : IVisitPlanService
    {
        private readonly IVisitStageRepository _visitStageRepository;
        private readonly IMapper _mapper;
        private readonly ISubjectVisitRepository _subjectVisitRepository;

        public VisitStageService(IVisitStageRepository visitStageRepository, IMapper mapper,ISubjectVisitRepository subjectVisitRepository)
        {
            _visitStageRepository = visitStageRepository;
            _mapper = mapper;
            _subjectVisitRepository = subjectVisitRepository;
        }

        ///暂时不用
        /// <summary> 获取项目访视计划</summary>
        public PageOutput<VisitStageDTO> GetTrialVisitStageList(VisitPlanQueryDTO param)
        {
            if (param.TrialId == Guid.Empty)
            {
                 ResponseOutput.NotOk("Trial Id should not be empty",new PageOutput<VisitStageDTO>());
            }
            Expression<Func<VisitStage, bool>> lambda = x => true;

            if (!string.IsNullOrWhiteSpace(param.Keyword))
            {
                lambda = lambda.And(t => t.VisitName.Contains(param.Keyword));
            }
            var query = from interview in _visitStageRepository.Find(lambda).Where(u => u.TrialId == param.TrialId)
                        select new VisitStageDTO
                        {
                            Id = interview.Id,
                            TrialId = interview.TrialId,
                            //PlanTime = interview.PlanTime,
                            VisitName = interview.VisitName,
                            VisitDay = interview.VisitDay,
                            Description = interview.Description,
                            CreateTime = interview.CreateTime
                        };

            var count = query.Count();
       
            query = param.Asc
                ? query.OrderBy("CreateTime")
                : query.OrderByDescending("CreateTime");

            query = query.Skip((param.PageIndex - 1) * param.PageSize).Take(param.PageSize);
            var list = query.ToList();

            return new PageOutput<VisitStageDTO>(param.PageIndex,
                param.PageSize, count, list);
           
        }

        public IEnumerable<VisitStageDTO> GetVisitStageList(Guid trialId)
        {
            var query =  _visitStageRepository.GetAll().Where(u => u.TrialId == trialId)
                .ProjectTo<VisitStageDTO>(_mapper.ConfigurationProvider).OrderBy(t => t.VisitNum);
            var list = query.ToList();

            return list;
        }
        public IEnumerable<VisitStageSelectDTO> GetTrialVisitStageSelect(Guid trialId)
        {
            var query = _visitStageRepository.GetAll().Where(u => u.TrialId == trialId)
                .ProjectTo<VisitStageSelectDTO>(_mapper.ConfigurationProvider).OrderBy(t => t.VisitDay);
            var list = query.ToList();
            return list;
        }


        /// <summary> 添加或更新项目访视阶段</summary>
        public IResponseOutput AddOrUpdateVisitStage(VisitPlanCommand param)
        {
            if (param.Id == Guid.Empty|| param.Id ==null)//add
            {
                var temp = _mapper.Map<VisitStage>(param);

                var result = _visitStageRepository.Add(temp);
                var success = _visitStageRepository.SaveChanges();

                return ResponseOutput.Result(success, new 
                {
                    NewId = result.Id
                });


            }
            else//update
            {
                var stage = _visitStageRepository.GetAll().FirstOrDefault(t => t.Id == param.Id);

                _mapper.Map(param, stage);
                _visitStageRepository.Update(stage);
                var isSuccess = _visitStageRepository.SaveChanges();

                return  ResponseOutput.Result(isSuccess) ;
            }

        }

        /// <summary> 删除项目访视计划</summary>
        public IResponseOutput DeleteVisitStage(Guid id)
        {
            var visitPlan = _visitStageRepository.GetAll().FirstOrDefault();

            if (_subjectVisitRepository.GetAll().Any(t => t.VisitNum == visitPlan.VisitNum))
            {
                return ResponseOutput.NotOk("Some subjects have the visit data");
            }

            var result = _visitStageRepository.Delete(u => u.Id == id);
            return ResponseOutput.Result(result);
        }

      
    }
}
